package de.mbutscher.wikiandpad;

import de.mbutscher.wikiandpad.db.DbReadAccessException;
import de.mbutscher.wikiandpad.db.DbWriteAccessException;
import de.mbutscher.wikiandpad.utils.StringOps;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Handler;
import android.text.ClipboardManager;
import android.widget.Toast;

public class ClipboardCatcher {
    
    protected Context context;
    protected Handler handler;
    protected ClipboardManager clipManager;
    protected String lastClipText;
    
    protected WikiPage wikiPage;
    protected int charOffset;
    
    //protected final static int POLL_DELAY = 5000; 

    protected Runnable onPoll = new Runnable() {
        public void run() {
            poll();
        }
    };
    
    public ClipboardCatcher(Context context) {
        this.context = context;
    }
    
    public synchronized boolean isActive() {
        return clipManager != null;
    }
    
    public synchronized void bindToPage(WikiPage wikiPage, int charOffset) {
        this.wikiPage = wikiPage;
        this.charOffset = charOffset;
    }
    
    public synchronized void start() {
        if (handler == null) {
            clipManager = (ClipboardManager) context.getSystemService(
                    Context.CLIPBOARD_SERVICE);
            handler = new Handler();
            
            clipManager.setText("");
            lastClipText = clipManager.getText().toString();
            
            SharedPreferences pref = WikiAndPadApplication.getAppPrefs();
            int pollDelay = Integer.parseInt(pref.getString(
                    "clipboardCatcher_pollDelay", "3000"));
            if (pollDelay < 100)
                pollDelay = 100;

            handler.postDelayed(onPoll, pollDelay);
        }
    }
    
    public synchronized void stop() {
        if (handler != null) {
            handler = null;
            clipManager = null;
            lastClipText = null;
            
            wikiPage = null;
            charOffset = -1;
        }
        
    }
    
    protected synchronized void poll() {
        if (handler == null)
            return;

        String newClipText = clipManager.getText().toString();
        if (newClipText != null) {
            if (lastClipText == null || (!lastClipText.equals(newClipText))) {
                onChange(newClipText);
            }
        }
        
        lastClipText = newClipText;

        SharedPreferences pref = WikiAndPadApplication.getAppPrefs();
        int pollDelay = Integer.parseInt(pref.getString(
                "clipboardCatcher_pollDelay", "3000"));
        if (pollDelay < 100)
            pollDelay = 100;

        handler.postDelayed(onPoll, pollDelay);
    }
    
    protected void onChange(String newClipText) {
        if (StringOps.notEmpty(newClipText) && wikiPage != null &&
                wikiPage.isValid()) {
            String liveText;
            try {
                liveText = wikiPage.getLiveText();
            } catch (DbReadAccessException e) {
                return;
            }
            
            String suffix = WikiAndPadApplication.getAppPrefs().getString(
                    "clipboardCatcher_suffix", "");
            
            suffix = StringOps.strftime(suffix, null);
            
            String before = liveText.substring(0, charOffset);
            String after = liveText.substring(charOffset);
            
            try {
                wikiPage.setLiveText(before + newClipText + suffix + after);
            } catch (DbWriteAccessException e) {
                return;
            }
            
            Toast.makeText(context,
                    String.format("Copied to WikidPad"),
                    Toast.LENGTH_SHORT).show();
            
            charOffset += newClipText.length() + suffix.length();
        }
        
    }
    
}
