package de.mbutscher.wikiandpad;

import de.mbutscher.wikiandpad.alphabeta.R;
import de.mbutscher.wikiandpad.utils.StringOps;
import de.mbutscher.wikiandpad.utils.ui.StringArrayListAdapter;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Bundle;
import android.text.ClipboardManager;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.ListView;


public class SnippetListActivity extends ListActivity {
    protected StringArrayListAdapter<String> listAdapter;

    public void updateList() {
        SharedPreferences pref = WikiAndPadApplication.getAppPrefs();
        String snippetStr = pref.getString("snippetCollector_snippets", null);

        if (StringOps.notEmpty(snippetStr))
            listAdapter.setListData(StringOps.strToStrList(snippetStr));
        else
            listAdapter.setListData(null);
    }



    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        setContentView(R.layout.snippet_list);
        
        listAdapter = new StringArrayListAdapter<String>(this, R.layout.snippet_list_item);

        getListView().setAdapter(listAdapter);
        /*
        Button btnOpenWiki = (Button) findViewById(R.id.btnOpenWiki);
        btnOpenWiki.setOnClickListener(onBtnOpenWiki);
        */

        registerForContextMenu(getListView());
    }
    

    @Override
    protected void onPause() {
        super.onPause();
    }


    @Override
    protected void onResume() {
        super.onResume();
        updateList();
    }
    
    /*
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.wiki_word_list, menu);
        return true;
    }
    
    public boolean onPrepareOptionsMenu (Menu menu) {
        MenuItem item = menu.findItem(R.id.mnCloseWiki);
        item.setVisible(WikiDocument.getActiveDocument() != null);
        
        return true;
    }
    */
    
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case R.id.mnPreferences:
            Intent intent = new Intent(getBaseContext(), Preferences.class);
            startActivity(intent);
            
            return false;
        default:
            return super.onOptionsItemSelected(item);
        }
    }
    
    @Override
    protected void onListItemClick(ListView l, View v, int position, long id) {
        String action = getIntent().getAction();
        
        if (action.equals(Intent.ACTION_PICK)) {
            Intent intent = new Intent(Intent.ACTION_PICK);
            intent.setComponent(new ComponentName(this, this.getClass()));
            intent.putExtra("snippet", listAdapter.get(position));
            
            setResult(RESULT_OK, intent);
            finish();
        }
        else if (action.equals(Intent.ACTION_MAIN)) {
            //View childView = l.getChildAt(position);
            //l.showContextMenuForChild(v);
            openContextMenu(v);
        }
    }

    @Override
    public void onCreateContextMenu(ContextMenu menu, View v,
            ContextMenuInfo menuInfo) {
        super.onCreateContextMenu(menu, v, menuInfo);
        if (v == getListView()) {
            MenuInflater inflater = getMenuInflater();
            inflater.inflate(R.menu.snippet_listitem_context, menu);
        }
    }

    @Override
    public boolean onContextItemSelected(MenuItem item) {
        AdapterContextMenuInfo info =
                (AdapterContextMenuInfo) item.getMenuInfo();
        switch (item.getItemId()) {
        case R.id.mnCopyClipboardThis:
            onCopyToClipboard(info.position);
            
            return true;
        case R.id.mnDeleteThis:
            onDelete(info.position);

            return true;
        case R.id.mnDeleteAll:
            onDeleteAll();

            return true;
            
        case R.id.mnSendThis:
            onCmdSend(info.position);

            return true;
        default:
            return super.onContextItemSelected(item);
        }
    }
    
    
    public void storeContent() {
        SharedPreferences pref = WikiAndPadApplication.getAppPrefs();
        Editor editor = pref.edit();
        if (listAdapter.isEmpty())
            editor.remove("snippetCollector_snippets");
        else {
            editor.putString("snippetCollector_snippets",
                    StringOps.strListToStr(listAdapter));
        }
        editor.commit();
    }

    
    public void onCopyToClipboard(final int position) {
        ClipboardManager clipManager = (ClipboardManager) getSystemService(
                Context.CLIPBOARD_SERVICE);
        clipManager.setText(listAdapter.get(position));
    }
    
    
    public void onCmdSend(final int position) {
        Intent intent = new Intent(Intent.ACTION_SEND);
        intent.setType("text/plain");

        intent.putExtra(Intent.EXTRA_TEXT, listAdapter.get(position));
        intent = Intent.createChooser(intent, "Send to");

        startActivity(intent);
    }



    public void onDelete(final int position) {
        AlertDialog.Builder alert = new AlertDialog.Builder(this);

        alert.setTitle("Delete snippet");
        // alert.setMessage("Message");

        alert.setPositiveButton("OK", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
                listAdapter.remove(position);
                storeContent();
                updateList();
            }
        });

        alert.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
                dialog.dismiss();
            }
        });

        alert.show();
    }
    
    public void onDeleteAll() {
        AlertDialog.Builder alert = new AlertDialog.Builder(this);

        alert.setTitle("Delete all snippets");
        // alert.setMessage("Message");

        alert.setPositiveButton("OK", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
                listAdapter.clear();
                storeContent();
                updateList();
            }
        });

        alert.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int whichButton) {
                dialog.dismiss();
            }
        });

        alert.show();
    }
    
}
