package de.mbutscher.wikiandpad;

import de.mbutscher.wikiandpad.db.MatchTermEntry;
import de.mbutscher.wikiandpad.utils.MiscEvent;
import de.mbutscher.wikiandpad.utils.MiscEventListener;
import android.content.ComponentName;
import android.content.Intent;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;

public class WikiWordEditActivity extends FragmentActivity
        implements MiscEventListener {
    
    protected WikiWordEditFragment wikiWordEditFragment;
    
    protected static final String EDIT_FRAGMENT_KEY = "WikiWordEditFragment";
    
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        if (savedInstanceState == null) {
            wikiWordEditFragment = new WikiWordEditFragment();
            getSupportFragmentManager().beginTransaction().add(
                    android.R.id.content,
                    wikiWordEditFragment, EDIT_FRAGMENT_KEY).commit();
        }
        else {
            wikiWordEditFragment = (WikiWordEditFragment)
                    getSupportFragmentManager().findFragmentByTag(
                    "WikiWordEditFragment");
        }
    }

    
    @Override
    protected void onPause() {
        MiscEvent.removeListener(wikiWordEditFragment, this);
        super.onPause();
    }

    @Override
    protected void onResume() {
        super.onResume();
        MiscEvent.addListener(wikiWordEditFragment, this);
    }

    
    public void miscEventHappened(MiscEvent event) {
        if (event.getSource() == wikiWordEditFragment) {
            if (event.hasKey("request finish"))
                finish();
            else if (event.hasKey("request openWikiWord")) {
                Intent intent = new Intent(Intent.ACTION_EDIT);
                intent.setComponent(new ComponentName(this,
                        WikiWordEditActivity.class));
                intent.putExtra("wikiConfigPath", (String) event.getValue(
                        "wikiConfigPath"));
                intent.putExtra("wikiPageName", (String) event.getValue(
                        "wikiPageName"));
                MatchTermEntry mte = event.getValue("matchTermEntry");
                
                if (mte != null) {
                    intent.putExtra("matchTermEntryBundle", mte.toBundle());
                }
                
                startActivity(intent);
            }
            else if (event.hasKey("request openPageStructure")) {
                PageStructureFragment pageStructure =
                        new PageStructureFragment();
                Bundle args = new Bundle();
                args.putString("editorFragmentKey",
                        EDIT_FRAGMENT_KEY);

                pageStructure.setArguments(args);

                //                         .detach(wikiWordEditFragment)

                getSupportFragmentManager().beginTransaction()
                        .hide(wikiWordEditFragment)
                        .add(android.R.id.content, pageStructure,
                                "PageStructureFragment")
                        .addToBackStack(null)
                        .commit();
            }
        }
    }

}
