package de.mbutscher.wikiandpad;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.IdentityHashMap;
import java.util.List;

import org.folder.FolderPicker;

import android.app.Activity;
import android.content.ActivityNotFoundException;
import android.content.ComponentName;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Typeface;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.text.Editable;
import android.text.Spanned;
import android.text.TextWatcher;
import android.text.style.ForegroundColorSpan;
import android.text.style.StyleSpan;
import android.text.style.UnderlineSpan;
import android.webkit.MimeTypeMap;
import android.widget.ToggleButton;
import android.view.ContextMenu;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import de.mbutscher.wikiandpad.alphabeta.R;
import de.mbutscher.wikiandpad.db.DbReadAccessException;
import de.mbutscher.wikiandpad.db.DbWriteAccessException;
import de.mbutscher.wikiandpad.db.MatchTermEntry;
import de.mbutscher.wikiandpad.db.WikiDataException;
import de.mbutscher.wikiandpad.parsing.NonTerminalNode;
import de.mbutscher.wikiandpad.parsing.SyntaxNode;
import de.mbutscher.wikiandpad.utils.MiscEvent;
import de.mbutscher.wikiandpad.utils.MiscEventListener;
import de.mbutscher.wikiandpad.utils.MiscEventSource;
import de.mbutscher.wikiandpad.utils.StringOps;
import de.mbutscher.wikiandpad.utils.Utilities;
import de.mbutscher.wikiandpad.utils.ui.ClickHandler;
import de.mbutscher.wikiandpad.utils.ui.FragmentGuiHelper;

public class WikiWordEditFragment extends Fragment implements MiscEventListener,
        MiscEventSource {
    
    protected FragmentGuiHelper<WikiWordEditActivity,
            WikiWordEditFragment> guiHelper;

    protected String wikiPageName;
    protected String wikiConfigPath;
    protected WikiPage wikiPage;
    protected boolean editorDirty;
    protected boolean hasSoftInputOnResume;
    protected List<SyntaxNode> contextMenuTokens;
    protected Runnable runAfterOnResume = null;
    protected MatchTermEntry matchTermEntryOnCreate = null;
    
    protected IdentityHashMap<Object, Object> usedSpans;
    
    protected ScrollingEditText edWikiWordContent;

    protected static final int REQUEST_PICK_SNIPPET = 1;
    

    protected TextWatcher dirtyWatcher = new TextWatcher() {
        public void afterTextChanged(Editable s) {
            editorDirty = true;
            
            if (wikiPage != null)
                wikiPage.setEditorText(s.toString());
        }

        public void beforeTextChanged(CharSequence s, int start, int count,
                int after) {
        }

        public void onTextChanged(CharSequence s, int start, int before,
                int count) {
        }
    };
    

    protected final android.content.DialogInterface.OnClickListener
            onInsertAbsoluteUrlPicked =
            new android.content.DialogInterface.OnClickListener() {
        public void onClick(android.content.DialogInterface dialog,
                int which) {

            String path = ((FolderPicker) dialog).getPath();
            String url = "file://" + StringOps.urlFromPathname(path);
            replaceTextAreaByCharPos(url,
                    edWikiWordContent.getSelectionLow(),
                    edWikiWordContent.getSelectionEnd());
        }
    };
    

    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        wikiPageName = getActivity().getIntent().getStringExtra("wikiPageName");

        /*
        if (wikiPageName == null) {
            wikiPageName = savedInstanceState != null ? savedInstanceState
                    .getString("wikiPageName") : null;
        }
        */
        
        if (wikiPageName == null) {
            getMiscEvent().cloneIt().put("request finish").send();
            return;
        }

        wikiConfigPath = getActivity().getIntent().getStringExtra(
                "wikiConfigPath");

        /*
        if (wikiConfigPath == null) {
            wikiConfigPath = savedInstanceState != null ? savedInstanceState
                    .getString("wikiConfigPath") : null;
        }
        */
        
        if (wikiConfigPath == null) {
            getMiscEvent().cloneIt().put("request finish").send();
            return;
        }

        Bundle mteBundle = getActivity().getIntent().getBundleExtra(
                "matchTermEntryBundle");
        if (mteBundle != null) {
            matchTermEntryOnCreate = new MatchTermEntry(mteBundle);
        }
        
        wikiPage = null;
        try {
            wikiPage = WikiDocument.openDocument(wikiConfigPath, false)
                    .getWikiPage(wikiPageName);
        } catch (FileNotFoundException e) {
        } catch (IOException e) {
        } catch (WikiDataException e) {
        }
        
        if (wikiPage == null) {
            getMiscEvent().cloneIt().put("request finish").send();
            return;
        }

        wikiPage.informVisited();

        setHasOptionsMenu(true);
    }
        
        
        
    @Override
    public View onCreateView(LayoutInflater inflater,
            ViewGroup container, Bundle savedInstanceState) {

        guiHelper = new FragmentGuiHelper<WikiWordEditActivity,
                WikiWordEditFragment>(this);
        
        guiHelper.setContentView(R.layout.wiki_word_edit, inflater,
                container, savedInstanceState);
        
        guiHelper.rewriteIfLandscape(guiHelper.findViewById(R.id.menuBar));

        // TODO: setTitle("Edit " + wikiPageName);

        edWikiWordContent = (ScrollingEditText) guiHelper.findViewById(
                R.id.edWikiWordContent);
        
        hasSoftInputOnResume = edWikiWordContent.hasSoftInput();

        guiHelper.setSubBarIds(Utilities.buildHashSet(R.id.barMove, R.id.barEdit,
                R.id.barInsert));
        guiHelper.hideSubBars();

        if (savedInstanceState != null) {
            edWikiWordContent.scrollTo(
                    savedInstanceState.getInt("edWikiWordContentScrollX"),
                    savedInstanceState.getInt("edWikiWordContentScrollY"));
            guiHelper.loadSubBarVisibility(savedInstanceState);
        }
        edWikiWordContent.addTextChangedListener(dirtyWatcher);
        edWikiWordContent.getMiscEvent().addListener(this);
        
        registerForContextMenu(edWikiWordContent);
        
        
        //guiHelper.setOnClickListener(R.id.btnBack, "onCmdBack");
        
        guiHelper.setAllClickHandlers(this);
        
        guiHelper.setOnClickListener(R.id.btnBarMove,
                guiHelper.new OnClickShowSubBar(R.id.barMove));
        guiHelper.setOnClickListener(R.id.btnCloseBarMove,
                guiHelper.onBtnHideSubBars);

        guiHelper.setOnClickListener(R.id.btnBarEdit, 
                guiHelper.new OnClickShowSubBar(R.id.barEdit));
        guiHelper.setOnClickListener(R.id.btnCloseBarEdit,
                guiHelper.onBtnHideSubBars);
//         guiHelper.setOnClickListener(R.id.btnSend,
//                 "onCmdSend");

        guiHelper.setOnClickListener(R.id.btnBarInsert, 
                guiHelper.new OnClickShowSubBar(R.id.barInsert));
        guiHelper.setOnClickListener(R.id.btnCloseBarInsert,
                guiHelper.onBtnHideSubBars);

        guiHelper.getActivity().getWindow().setSoftInputMode(
                WindowManager.LayoutParams.SOFT_INPUT_STATE_HIDDEN);

        usedSpans = new IdentityHashMap<Object, Object>();
        
        return guiHelper.getContentView();
    }


    @Override
    public void onPause() {
        super.onPause();
        
        wikiPage.getMiscEvent().removeListener(this);

        if (editorDirty) {
            try {
                wikiPage.writeToDatabase(edWikiWordContent.getText().toString(),
                        true);
            } catch (DbWriteAccessException e) {
                // TODO: What?
            }
        }
        wikiPage.setEditorText(null);
        
        wikiPage = null;
    }

    @Override
    public void onResume() {
        super.onResume();
        
        // WikiAndPadApplication.getApp().stopClipboardCatcher();

        ((ToggleButton) guiHelper.findViewById(R.id.tgBtnClipboardCatcher))
                .setChecked(WikiAndPadApplication.getApp()
                .getClipboardCatcher().isActive());
        
        
        wikiPage = null;
        try {
            wikiPage = WikiDocument.openDocument(wikiConfigPath, false)
                    .getWikiPage(wikiPageName);
        } catch (FileNotFoundException e) {
        } catch (IOException e) {
        } catch (WikiDataException e) {
        }
        
        if (wikiPage == null) {
            getMiscEvent().cloneIt().put("request finish").send();
            return;
        }

        wikiPage.getMiscEvent().addListener(this);
        
        try {
            wikiPage.setEditorText(wikiPage.getWikiPageContent());
            edWikiWordContent.setTextKeepState(wikiPage.getEditorText());
        } catch (DbReadAccessException e) {
            edWikiWordContent.setText("");
        }
        editorDirty = false;
        
        // Select text as described in the entry
        if (matchTermEntryOnCreate != null) {
            setSelection(matchTermEntryOnCreate,
                    ScrollingEditText.ScrollTo.MIDDLE);

            matchTermEntryOnCreate = null;
        }
        
        if (runAfterOnResume != null) {
            runAfterOnResume.run();
            runAfterOnResume = null;
        }
        
        // Log.d("WikiAndPad", pageAst.pprint());

    }


    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        
        outState.putInt("edWikiWordContentScrollX",
                edWikiWordContent.getScrollX());
        outState.putInt("edWikiWordContentScrollY",
                edWikiWordContent.getScrollY());
        
        guiHelper.saveSubBarVisibility(outState);
    }
    
    
    public int getTextCursorPos() {
        return edWikiWordContent.getSelectionStart();
    }
    
    /**
     * Set selection by position data in MatchTermEntry
     * @param mte
     * @param scrollTo One of the {@link ScrollingEditText}.SCROLL_TO_* constants
     */

    public void setSelection(MatchTermEntry mte,
            ScrollingEditText.ScrollTo scrollTo) {
        assert mte != null;
        edWikiWordContent.setSelection(mte.firstcharpos,
                mte.firstcharpos + mte.charlength, scrollTo);
    }

    
    
    
    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        inflater.inflate(R.menu.wiki_word_edit, menu);
        super.onCreateOptionsMenu(menu, inflater);
    }


    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case R.id.mnPreferences:
            Intent intent = new Intent(getActivity().getBaseContext(),
                    Preferences.class);
            startActivity(intent);
            
            return false;
        case R.id.mnSearch:
            getActivity().onSearchRequested();
            
            return false;
        /*
        case R.id.mnInsertAbsoluteUrl:
            onCmdInsertAbsoluteUrl();
            return false;
        */
        default:
            return super.onOptionsItemSelected(item);
        }
    }

    @Override
    public void onCreateContextMenu(ContextMenu menu, View v,
            ContextMenu.ContextMenuInfo menuInfo) {
        super.onCreateContextMenu(menu, v, menuInfo);
        
        boolean fileurl = false;
        
        SyntaxNode pageAst = wikiPage.getLivePageAst();
                
        contextMenuTokens = pageAst.findNodesForCharPos(
                edWikiWordContent.getSelectionStart());
        
        for(SyntaxNode node: contextMenuTokens) {
            if ("urlLink".equals(node.getName())) {
                String url = (String) node.dGet("url");
                
                if (url == null)
                    continue;
                
                if (url.startsWith("file:") || url.startsWith("rel:") ||
                        url.startsWith("wiki:") || url.startsWith("wikirel:")) {
                    fileurl = true;
                }
            }
        }
        
        if (fileurl) {
            MenuInflater inflater = getActivity().getMenuInflater();
            inflater.inflate(R.menu.wiki_word_edit_fileurl_context, menu);
        }
        
        if (edWikiWordContent.getSelectionStart() !=
                edWikiWordContent.getSelectionEnd()) {
            MenuInflater inflater = getActivity().getMenuInflater();
            inflater.inflate(R.menu.wiki_word_edit_selection_context, menu);
        }

    }


    @Override
    public boolean onContextItemSelected (MenuItem item) {
        switch (item.getItemId()) {
        case R.id.mnFileUrlConvertAbsRel:
            convertUrlAbsoluteRelative(contextMenuTokens);
            return true;
        case R.id.mnSelectionSend:
            onCmdSend();
            return true;
        default:
            return super.onContextItemSelected(item);
        }
    }
    
    
    /**
     * Must be called in UI thread.
     * @param spanList
     */
    public synchronized void applySpans(List<SpanEntry> spanList) {
        Editable editable = edWikiWordContent.getEditableText();
        
        edWikiWordContent.beginBatchEdit();
        try {
            for (Object span: usedSpans.keySet()) {
                editable.removeSpan(span);
            }

            usedSpans.clear();
            for (SpanEntry entry: spanList) {
                usedSpans.put(entry.object, null);
            }

            SpanEntry.applyList(spanList, editable);
            edWikiWordContent.invalidate();
        }
        finally {
            edWikiWordContent.endBatchEdit();
        }
    }
    
    protected void recursConvertAstToSpanList(NonTerminalNode pageAst,
            ArrayList<SpanEntry> result) {
        WikiDocument wikiDoc = wikiPage.getWikiDocument();
        
        for(SyntaxNode node: pageAst.iterFlatNamed(0)) {
            String name = node.getName();
            
            if ("bold".equals(name)) {
                result.add(new SpanEntry(new StyleSpan(Typeface.BOLD),
                        node.getPos(), node.getPos() + node.getStrLength(),
                        Spanned.SPAN_EXCLUSIVE_INCLUSIVE));
                recursConvertAstToSpanList((NonTerminalNode) node, result);
            }
            else if ("italics".equals(name)) {
                result.add(new SpanEntry(new StyleSpan(Typeface.ITALIC),
                        node.getPos(), node.getPos() + node.getStrLength(),
                        Spanned.SPAN_EXCLUSIVE_INCLUSIVE));
                recursConvertAstToSpanList((NonTerminalNode) node, result);
            }
            else if ("wikiWord".equals(name)) {
                if (!wikiDoc.isCreatableWikiWord(
                        (String) node.dGet("wikiWord"))) {
                    // Existing wiki word -> blue
                    result.add(new SpanEntry(new ForegroundColorSpan(0xFF0000FF),
                            node.getPos(), node.getPos() + node.getStrLength(),
                            Spanned.SPAN_EXCLUSIVE_INCLUSIVE));
                }
                result.add(new SpanEntry(new UnderlineSpan(),
                        node.getPos(), node.getPos() + node.getStrLength(),
                        Spanned.SPAN_EXCLUSIVE_INCLUSIVE));
            }
            else if ("urlLink".equals(name)) {
                result.add(new SpanEntry(new ForegroundColorSpan(0xFF0000FF),
                        node.getPos(), node.getPos() + node.getStrLength(),
                        Spanned.SPAN_EXCLUSIVE_INCLUSIVE));

                result.add(new SpanEntry(new UnderlineSpan(),
                        node.getPos(), node.getPos() + node.getStrLength(),
                        Spanned.SPAN_EXCLUSIVE_INCLUSIVE));
            }
            else if ("heading".equals(name)) {
                result.add(new SpanEntry(new StyleSpan(Typeface.BOLD),
                        node.getPos(), node.getPos() + node.getStrLength(),
                        Spanned.SPAN_EXCLUSIVE_INCLUSIVE));
                recursConvertAstToSpanList((NonTerminalNode) node, result);
            }
        }
    }
    
    public List<SpanEntry> convertAstToSpanList(SyntaxNode pageAst) {
        ArrayList<SpanEntry> result = new ArrayList<SpanEntry>();
        
        recursConvertAstToSpanList((NonTerminalNode) pageAst, result);
        
        return result;
    }


    public void miscEventHappened(MiscEvent event) {
        if (event.getSource() == wikiPage) {
            if (event.hasKey("updated livePageAst")) {
                final Object eventLiveText = event.getValue("liveTextForPageAst");
                if (wikiPage.getEditorText() == eventLiveText) {
                    final List<SpanEntry> spanEntries = convertAstToSpanList(
                            (SyntaxNode) event.getValue("livePageAst"));

                    getActivity().runOnUiThread(new Runnable()  {
                        public void run() {
                            if (wikiPage.getEditorText() == eventLiveText)
                                applySpans(spanEntries);
                        }
                    });
                }
            }
        }
        else if (event.getSource() == edWikiWordContent) {
            if (event.hasKey("activated text")) {
                SyntaxNode pageAst = wikiPage.getLivePageAst();
                int charPos = (Integer) event.getValue("textOffset");
                
                final List<SyntaxNode> nodes = pageAst.findNodesForCharPos(
                        charPos);
                if (onSyntaxNodeActivation(nodes))
                    event.put("processed");
            }
        }
    }


    public boolean onSyntaxNodeActivation(List<SyntaxNode> nodes) {
        for(SyntaxNode node: nodes) {
            if ("wikiWord".equals(node.getName())) {
                String wikiPageName = wikiPage.getWikiDocument()
                        .getWikiPageNameForLinkTermOrAsIs(
                        (String) node.dGet("wikiWord"));
                /*
                Intent intent = new Intent(Intent.ACTION_EDIT);
                intent.setComponent(new ComponentName(this,
                        WikiWordEditFragment.class));
                intent.putExtra("wikiPageName", wikiPageName);
                
                startActivity(intent);
                */
                
                getMiscEvent().cloneIt().put("request openWikiWord")
                        .put("wikiConfigPath",
                            wikiPage.getWikiDocument().getWikiConfigPath())
                        .put("wikiPageName", wikiPageName)
                        .send();

                return true;
            }
            else if ("urlLink".equals(node.getName())) {
                String url = (String) node.dGet("url");
                
                if (url == null)
                    continue;
                
                /*
                Log.d("WikiAndPad", "url1 " + url);
                Log.d("WikiAndPad", "url2 " +
                        Uri.fromFile(new File("/sdcard/Test.txt")).toString());
                */
                
                Intent intent;

                if (url.startsWith("rel:")) {
                    url = WikiDocument.getActiveDocument()
                            .makeRelUrlAbsolute(url);
                }

                if (url.startsWith("file:")) {
                    File f = WikiDocument.getActiveDocument()
                            .makeFileUrlAbsFile(url);
                
                /*
                }
                
                if ("file".equals(uri.getScheme())) {
                    String path = uri.getPath();
                    File f = new File(path);
                */
                    if (!f.exists()) {
                        guiHelper.alertInfo("Target doesn't exist");
                        return true;
                    }
                    
                    intent = new Intent(Intent.ACTION_VIEW);
                    String suffix = MimeTypeMap.getFileExtensionFromUrl(url);
                    String mimeType = MimeTypeMap.getSingleton()
                            .getMimeTypeFromExtension(suffix);
                    Uri uri = Uri.parse(url);
                    
                    intent.setDataAndType(uri, mimeType);
                    
                    // Log.d("WikiAndPad", "url3 " + f);
                }
                else {
                    Uri uri = Uri.parse(url);
                    intent = new Intent(Intent.ACTION_VIEW, uri);
                }
                
                try {
                    startActivity(intent);
                }
                catch(ActivityNotFoundException e) {
                    guiHelper.alertInfo("No application found for URL");
                }
                return true;
            }
        }
        
        return false;
    }
 
    
    public void convertUrlAbsoluteRelative(List<SyntaxNode> tokenList) {
        for(SyntaxNode node: tokenList) {
            if ("urlLink".equals(node.getName())) {
                String link = (String) node.dGet("url");
                
                if (link == null)
                    continue;
                
                // TODO: Implement
                
                /*
                if ' ' in node.coreNode.getString():
                    addSafe = ' '
                else:
                    addSafe = ''
                    
                */

                WikiDocument wikiDoc = wikiPage.getWikiDocument();
                
                if (link.startsWith("rel://") || link.startsWith("wikirel://")) {
                    link = wikiDoc.makeRelUrlAbsolute(link); // , addSafe=addSafe
                }
                else if (link.startsWith("file:")) {
                    link = wikiDoc.makeAbsPathRelUrl(StringOps.pathnameFromUrl(
                            link)); // , addSafe=addSafe
                    if (link == null)
                        continue; // TODO Message?
                }
                else if (link.startsWith("wiki:")) {
                    link = wikiDoc.makeAbsPathRelUrl(StringOps.pathnameFromUrl(
                            link)); // , addSafe=addSafe
                    if (link == null)
                        continue; // TODO Message?
                    else
                        link = "wiki" + link;  // Combines to "wikirel://"
                }

                else
                    continue;

                SyntaxNode coreNode = (SyntaxNode) node.dGet("coreNode");
                
                replaceTextAreaByCharPos(link, coreNode.getPos(),
                        coreNode.getPos() + coreNode.getStrLength());

                break;
            }
        }
    }
    
    /**
     * Delete text between character offsets start and end and place
     * newText there instead
     * @param newText
     * @param start
     * @param end
     */
    public void replaceTextAreaByCharPos(String newText, int start,
            int end) {
        Editable text = edWikiWordContent.getText();
        text.replace(start, end, newText);
    }
    
    
    @ClickHandler(R.id.btnBack)
    public void onCmdBack() {
        if (!hasSoftInputOnResume) {
            ((ScrollingEditText) guiHelper.findViewById(
                    R.id.edWikiWordContent)).hideSoftInput();
        }
        getMiscEvent().cloneIt().put("request finish").send();
    }

    @ClickHandler(R.id.btnInsertAbsoluteUrl)
    public void onCmdInsertAbsoluteUrl() {
        SharedPreferences prefs = WikiAndPadApplication.getAppPrefs();
        
        ArrayList<String> paths = StringOps.strToStrList(prefs.getString(
                "lruWikiPaths", ""));
        
        FolderPicker fileDialog = new FolderPicker(
                getActivity(), onInsertAbsoluteUrlPicked,
                android.R.style.Theme, true, paths);
        fileDialog.show();
    }
    
    
    @ClickHandler(R.id.btnInsertSnippet)
    public void onCmdInsertSnippet() {
        Intent intent = new Intent(Intent.ACTION_PICK);
        intent.setComponent(new ComponentName(getActivity(),
                SnippetListActivity.class));
        
        startActivityForResult(intent, REQUEST_PICK_SNIPPET);
    }
    
    public void onCmdSend() {
        Intent intent = new Intent(Intent.ACTION_SEND);
        intent.setType("text/plain");
        
        String text = edWikiWordContent.getText().subSequence(
                edWikiWordContent.getSelectionLow(),
                edWikiWordContent.getSelectionHigh()).toString();

        intent.putExtra(Intent.EXTRA_TEXT, text);
        intent = Intent.createChooser(intent, "Send to");

        startActivity(intent);
    }


    @ClickHandler(R.id.btnCancelEdit)
    public void onCmdCancelEdit() {
        editorDirty = false;
        if (!hasSoftInputOnResume) {
            ((ScrollingEditText) guiHelper.findViewById(R.id.edWikiWordContent))
                    .hideSoftInput();
        }
        getMiscEvent().cloneIt().put("request finish").send();
    }


    @ClickHandler(R.id.btnList)
    public void onCmdGoToList() {
        if (!hasSoftInputOnResume) {
            ((ScrollingEditText) guiHelper.findViewById(R.id.edWikiWordContent))
                    .hideSoftInput();
        }

        Intent intent = new Intent(getActivity().getBaseContext(),
                WikiWordListActivity.class);
        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);

        startActivity(intent);
    }

    
    @ClickHandler(R.id.btnOpenPageStructure)
    public void onCmdOpenPageStructure() {
        getMiscEvent().cloneIt().put("request openPageStructure").send();
    }
    
    
    @ClickHandler(R.id.btnActivate)
    public void onCmdActivate() {
        edWikiWordContent.activateSelection();
    }

    @ClickHandler(R.id.tgBtnClipboardCatcher)
    protected void onTgBtnClipboardCatcher(View v) {
        ClipboardCatcher catcher = WikiAndPadApplication.getApp()
                .getClipboardCatcher();

        if (catcher.isActive()) {
            catcher.stop();
            ((ToggleButton) v).setChecked(false);
        } else {
            catcher.bindToPage(wikiPage, edWikiWordContent.getSelectionStart());
            catcher.start();
            ((ToggleButton) v).setChecked(true);
        }
    }

    
    
    /*
    public void onBtnBack(View v) {
        if (!hasSoftInputOnResume) {
            ((ScrollingEditText) findViewById(
                    R.id.edWikiWordContent)).hideSoftInput();
        }
        finish();
    };
    */
    
    public WikiPage getCurrentWikiPage() {
        return wikiPage;
    }
    

    @Override
    public void onActivityResult(int requestCode, int resultCode,
            Intent data) {
        if (requestCode != REQUEST_PICK_SNIPPET)
            return;
        if (resultCode != Activity.RESULT_OK)
            return;
        
        final String textToInsert = data.getStringExtra("snippet");
        
       
        final int start = edWikiWordContent.getSelectionLow();
        final int end = edWikiWordContent.getSelectionHigh();
        
        runAfterOnResume = new Runnable() {
            public void run() {
                replaceTextAreaByCharPos(textToInsert, start, end);
            }
        };

        //edWikiWordContent.getText().replace(Math.min(start, end),
        //        Math.max(start, end), textToInsert, 0, textToInsert.length());
    }


    protected MiscEvent miscEvent = new MiscEvent(this);
    
    public MiscEvent getMiscEvent() {
        return miscEvent;
    }

    
}
