package de.mbutscher.wikiandpad.parsing;

/**
 * Abstract class providing factory methods for common syntax elements
 * @author Michael Butscher
 *
 */
public abstract class AbstractParserGrammar {

    public static Choice choice(ParserElement[] exprs,
            ParseChoiceAction parseChoiceAction) {
        return new Choice(exprs, parseChoiceAction);
    }

    public static FindFirst findFirst(ParserElement[] exprs,
            ParserElement endExpr) {
        return new FindFirst(exprs, endExpr);
    }

    public static FollowedBy followedBy(ParserElement expr) {
        return new FollowedBy(expr);
    }

    public static Forward forward() {
        return new Forward();
    }

    public static MatchFirst matchFirst(ParserElement... exprs) {
        return new MatchFirst(exprs);
    }

    public static NotAny notAny(ParserElement expr) {
        return new NotAny(expr);
    }

    public static OneOrMore oneOrMore(ParserElement expr) {
        return new OneOrMore(expr);
    }

    public static Optional optional(ParserElement expr) {
        return new Optional(expr);
    }

    public static Regex regex(String pattern, int flags) {
        return new Regex(pattern, flags);
    }

    public static StringEnd stringEnd() {
        return new StringEnd();
    }

    public static ZeroOrMore zeroOrMore(ParserElement expr) {
        return new ZeroOrMore(expr);
    }

}
