package de.mbutscher.wikiandpad.parsing;

import java.util.ArrayList;
import java.util.regex.Pattern;

import de.mbutscher.wikiandpad.utils.NotCurrentThreadException;


/**
 * Requires all given ParseExpressions to be found in the given order.
 * May be constructed using the {@link ParserElement#and(ParserElement)} method.
 * 
 */
public class And extends ParseExpression implements NecessaryRegexProvider {
    
    protected int reFlagsMask;
    protected boolean reComplete;
    
    
    
    public And(ParserElement[] exprs) {
        super(exprs);
        mayReturnEmpty = true;
        for(ParserElement e: exprs) {
            if (!e.mayReturnEmpty) {
                mayReturnEmpty = false;
                break;
            }
        }
        
        reFlagsMask = 0;
        reComplete = false;
    }

    public Pattern getRegex() {
        if (buildingRegex)
            return null;
        
        buildingRegex = true;
        try {
            reFlagsMask = 0;
            reComplete = false;
            
            StringBuilder result = new StringBuilder();
            int flags = 0;
            int flagsMask = 0;
            boolean complete = true;
            
            for (ParserElement pe: exprs) {
                if (!(pe instanceof NecessaryRegexProvider)) {
                    complete = false;
                    break;
                }
                
                NecessaryRegexProvider e = (NecessaryRegexProvider) pe;
                                
                Pattern nextRe = e.getRegex();
                if (nextRe == null) {
                    complete = false;
                    break;
                }
                
                int[] comb = RegexCombiner.combineRegexFlags(flags, flagsMask,
                        nextRe.flags(), e.getRegexFlagsMask());
                int nextFlags = comb[0];
                int nextFlagsMask = comb[1];
                
                if (nextFlags == -1) {
                    complete = false;
                    break;
                }
                
                result.append("(?:" + nextRe.pattern() + ")");
                flags = nextFlags;
                flagsMask = nextFlagsMask;
                complete = complete && e.isRegexComplete();
                
                if (!complete) {
                    // If the current regex isn't complete, further parts can't
                    // be appended to the and-regex
                    break;
                }
                
            }
            if (result.length() == 0)
                return null;
            
            this.reFlagsMask = flagsMask;
            this.reComplete = complete;
            
            return Pattern.compile(result.toString(), flags);
            
        }
        finally {
            buildingRegex = false;
        }
    }
    
    public int getRegexFlagsMask() {
        return reFlagsMask;
    }

    public boolean isRegexComplete() {
        return reComplete;
    }

    @Override
    protected ParsingResult parseImpl(String instring, int loc,
            ParsingState state, boolean doActions) throws ParseException,
            NotCurrentThreadException {
    
        ArrayList<SyntaxNode> resultlist = new ArrayList<SyntaxNode>();
        ParsingResult parsingResult;
    
        for(ParserElement e: exprs) {
            parsingResult = e.parse(instring, loc, state, doActions);
            loc = parsingResult.nextLoc;
            resultlist.addAll(parsingResult.retTokens);
        }
    
        return new ParsingResult(loc, resultlist);
    }

}
