package de.mbutscher.wikiandpad.parsing;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


public class BaseParsingProduct {
    public List<SyntaxNode> retTokens;
    public boolean fromSingleNode;
    
    public static final List<SyntaxNode> EMPTYLIST = Arrays.asList(
            new SyntaxNode[0]);

    public BaseParsingProduct(List<SyntaxNode> retTokens) {
        setRetTokens(retTokens);
    }
    
    public BaseParsingProduct(SyntaxNode syntaxNode) {
        setRetTokens(syntaxNode);
    }

    /**
     * Necessary to resolve ambiguity because NonTerminalNode implements
     * List&lt;SyntaxNode&gt;
     */
    public BaseParsingProduct(NonTerminalNode syntaxNode) {
        this((SyntaxNode) syntaxNode);
    }
    
    
    public BaseParsingProduct(int loc) {
        this(EMPTYLIST);
    }

    public BaseParsingProduct() {
        this(EMPTYLIST);
    }

    public void setRetTokens(List<SyntaxNode> retTokens) {
        if (retTokens == null)
            this.retTokens = EMPTYLIST;
        else
            this.retTokens = retTokens;

        this.fromSingleNode = false;
    }
    
    public void setRetTokens(SyntaxNode syntaxNode) {
        if (syntaxNode == null) {
            this.retTokens = EMPTYLIST;
            fromSingleNode = false;
            return;
        }
        
        ArrayList<SyntaxNode> rt = new ArrayList<SyntaxNode>();
        rt.add(syntaxNode);

        this.retTokens = rt;
        fromSingleNode = true;
    }

/*    
    public SyntaxNode getFirstToken() {
        if (retTokens == null || retTokens.size() == 0)
            return null;
        
        if (retTokens.get(0) == null)
            Log.d("WikiAndPad", "Error");
        
        return retTokens.get(0);
    }
    
    public NonTerminalNode getFirstTokenNt() {
        return (NonTerminalNode) getFirstToken();
    }

    public TerminalNode getFirstTokenT() {
        return (TerminalNode) getFirstToken();
    }
*/
    
    public void addToken(SyntaxNode node) {
        if (node == null)
            return;
        
        if (retTokens == null) {
            retTokens = Arrays.asList(node);
            return;
        }

        if (!(retTokens instanceof ArrayList<?>))
            retTokens = new ArrayList<SyntaxNode>(retTokens);
        
        retTokens.add(node);
    }

    
    public void addAllTokens(List<SyntaxNode> otherList) {
        if (otherList == null || otherList.isEmpty())
            return;
        
        if (retTokens == null) {
            retTokens = new ArrayList<SyntaxNode>(otherList);
            return;
        }

        if (!(retTokens instanceof ArrayList<?>))
            retTokens = new ArrayList<SyntaxNode>(retTokens);
        
        retTokens.addAll(otherList);
    }
    

    public void addAllTokens(BaseParsingProduct otherPr) {
        if (otherPr == null)
            return;

        addAllTokens(otherPr.retTokens);
    }
    

    /*
    public ParsingResult(List<SyntaxNode> retTokens) {
        this(-1, retTokens);
    }

    public ParsingResult(SyntaxNode syntaxNode) {
        this(-1, syntaxNode);
    }
    */

}
