package de.mbutscher.wikiandpad.parsing;

import java.util.regex.Pattern;

import de.mbutscher.wikiandpad.utils.NotCurrentThreadException;
import de.mbutscher.wikiandpad.utils.StringOps;
import de.mbutscher.wikiandpad.utils.Utilities;

public class Choice extends ParseExpression implements NecessaryRegexProvider {

    protected ParseChoiceAction parseChoiceAction;
    protected RegexCombiner regexCombiner;
    
    public Choice(ParserElement[] exprs, ParseChoiceAction parseChoiceAction) {
        super(exprs);
        if (exprs.length > 0) {
            mayReturnEmpty = false;
            for(ParserElement e: exprs) {
                if (e.mayReturnEmpty) {
                    mayReturnEmpty = true;
                    break;
                }
            }
        }
        else
            mayReturnEmpty = true;
    
        this.parseChoiceAction = parseChoiceAction;
    }

    
    @Override
    protected ParsingResult parseImpl(String instring, int loc,
            ParsingState state, boolean doActions) throws ParseException,
            NotCurrentThreadException {
        
        if (parseChoiceAction == null || exprs == null || exprs.size() == 0) {
            throw new ParseException(instring,
                    instring.length(),
                    "Choice: No choice action or no expressions given", this);
        }
        
        ParserElement expr = parseChoiceAction.parseChoiceAction(instring, loc,
                state, this);
        if (expr == null) {
            throw new ParseException(instring, instring.length(),
                    "Choice: Choice action returned None", this);
        }

        return expr.parse(instring, loc, state, doActions);
    }

    public Pattern getRegex() {
        if (regexCombiner != null) {
            return Pattern.compile(regexCombiner.getCleanPattern(),
                    regexCombiner.getFlags());
        }
        return null;
    }


    public int getRegexFlagsMask() {
        if (regexCombiner == null) 
            return 0;

        return regexCombiner.getFlagsMask();
    }


    public boolean isRegexComplete() {
        return false;
    }

    
    @Override
    protected void _optimizeSub(OptimizerOptions options) {
        super._optimizeSub(options);
        
        if (regexCombiner != null || !options.regexcombine)
            return;
        
        RegexCombiner combiner = new RegexCombiner(exprs,
                RegexCombiner.REMODE_MATCH);
        if (combiner.combine())
            this.regexCombiner = combiner;
        
    }

            
            
    @Override
    public String toString() {
        return "Choice(" + StringOps.strObjJoin(" | ", Utilities.castList(exprs))
                + ")";
    }

}


