package de.mbutscher.wikiandpad.parsing;

import java.util.ArrayList;
import java.util.List;

import de.mbutscher.wikiandpad.utils.NotCurrentThreadException;
import de.mbutscher.wikiandpad.utils.StringOps;

public class FindFirst extends ParseExpression {

    protected ParserElement endExpr;
    protected List<ParseAction> pseudoParseActions;
    protected RegexCombiner regexCombinerAll;


    public FindFirst(ParserElement[] exprs, ParserElement endExpr) {
        super(exprs);
        this.endExpr = endExpr;
        
        if (endExpr != null)
            mayReturnEmpty = true;
        else if (exprs.length > 0) {
            mayReturnEmpty = false;
            for(ParserElement e: exprs) {
                if (e.mayReturnEmpty) {
                    mayReturnEmpty = true;
                    break;
                }
            }
        }
        else
            mayReturnEmpty = true;
    }

    public FindFirst setPseudoParseAction(ParseAction pa) {
        pseudoParseActions = new ArrayList<ParseAction>(1);
        pseudoParseActions.add(pa);

        return this;
    }

    public FindFirst addPseudoParseAction(ParseAction pa) {
        pseudoParseActions.add(pa);
                
        return this;
    }

    public RegexCombiner getRegexCombiner() {
        return regexCombinerAll;
    }

    @Override
    protected ParsingResult parseImpl(String instring, int loc,
            ParsingState state, boolean doActions) throws ParseException,
            NotCurrentThreadException {

        int instrlen = instring.length();
        ParsingResult parsingResult = null;
        boolean endTokenFound = false;
        int startLoc = loc;

        if (regexCombinerAll == null) {
            while (loc <= instrlen) {
                try {
                    parsingResult = endExpr.parse(instring, loc, state, doActions);
                    endTokenFound = true;
                }
                catch (ParseException err) {
                    for(ParserElement e: exprs) {
                        try {
                            parsingResult = e.parse(instring, loc, state,
                                    doActions);
                            break;
                        }
                        catch (ParseException err2) {}
                        catch (ArrayIndexOutOfBoundsException err2) {}
                    }
                }
                catch (ArrayIndexOutOfBoundsException err) {
                    // Copy of  catch (ParseException err)  block
                    for(ParserElement e: exprs) {
                        try {
                            parsingResult = e.parse(instring, loc, state,
                                    doActions);
                            break;
                        }
                        catch (ParseException err2) {}
                        catch (ArrayIndexOutOfBoundsException err2) {}
                    }
                }

                if (parsingResult != null)
                    break;

                loc += 1;
            }
        }
        else {
            while (loc <= instrlen) {
                RegexCombiner.FindAllResult findAll = regexCombinerAll.findAll(
                        instring, loc);
                
                /* If idx is 0, the end expression matched, higher values
                 * matched self.exprs[idx-1]
                 * idx == -1 means nothing found
                 */
                
                if (findAll.styles.isEmpty()) {
                    throw new ParseException(instring, startLoc, String.format(
                            "Neither end %s nor one of %s found",
                            endExpr, exprs), this);
                }
                
                List<Integer> styles = findAll.styles;
                loc = findAll.loc;
                
                if (styles.get(0) == 0) {
                    try {
                        parsingResult = endExpr.parse(instring, loc, state,
                                doActions);
                        endTokenFound = true;
                        break;
                    }
                    catch (ParseException err) {
                        styles.remove(0);
                        if (styles.isEmpty()) {
                            loc += 1;
                            continue;
                        }
                    }
                    catch (ArrayIndexOutOfBoundsException err) {
                        styles.remove(0);
                        if (styles.isEmpty()) {
                            loc += 1;
                            continue;
                        }
                    }
                }
                    
                for (int st: styles) {
                    try {
                        parsingResult = exprs.get(st - 1).parse(instring, loc,
                                state, doActions);
                        break;
                    }
                    catch (ParseException err2) {}
                    catch (ArrayIndexOutOfBoundsException err2) {}
                }
                
                if (parsingResult != null)
                    break;
                
                loc += 1;
            }
        }

        if (parsingResult == null) {
            throw new ParseException(instring, startLoc,
                    String.format("Neither end %s nor one of %s found",
                    endExpr, exprs), this);
        }
        
        // List<SyntaxNode> pTokens = null;
        
        SyntaxNode pseudoToken = null;

        for(ParseAction e: pseudoParseActions) {
            SyntaxNode helperToken = SyntaxNode.buildSyntaxNode(
                    instring.substring(startLoc,
                    loc), startLoc);

            pseudoToken = e.parseAction(instring, startLoc, state, helperToken);
        }

        //if (pseudoPpr == null)
        //    pseudoPpr = new ParseProcessingData();

        //pseudoPpr.fromSingleNode = false;
        
        ParsingResult result;

        if (pseudoToken != null) {
            if (StringOps.notEmpty(pseudoToken.name))
                result = new ParsingResult(loc, pseudoToken);
            else
                result = new ParsingResult(loc, pseudoToken.asList());
        }
        else
            result = new ParsingResult(loc);
        

        if (endTokenFound) {
            result.nextLoc = loc;
        }
        else {
            result.addAllTokens(parsingResult);
            result.nextLoc = parsingResult.nextLoc;
        }
        
        return result;
    }
    
    
    @Override
    protected void _optimizeSub(OptimizerOptions options) {
        super._optimizeSub(options);
        endExpr._optimizeSub(options);

        if (regexCombinerAll != null || !options.regexcombine)
            return;

        ArrayList<ParserElement> combEx = new ArrayList<ParserElement>(
                exprs.size() + 1);
        
        combEx.add(endExpr);
        combEx.addAll(exprs);
        
        
        RegexCombiner combiner = new RegexCombiner(combEx,
                RegexCombiner.REMODE_SEARCH_ALL);

        if (combiner.combine())
            regexCombinerAll = combiner;
    }

    
}
