package de.mbutscher.wikiandpad.parsing;

import java.util.regex.Pattern;

/**
 * Classes implementing this may provide a regex which must necessarily match
   to match the parser element. If getRegex() returns null, such a regex
   can't be provided for this particular object.
 * @author Michael Butscher
 *
 */
public interface NecessaryRegexProvider {
        
        public Pattern getRegex();

        /**
         * Return bitmask with only the relevant flags.
         * @return
         */
        public int getRegexFlagsMask();

        /**
         * Returns True if the regex does not only provide a necessary match but
            the complete match of the parser element. Even if this returns True,
            there may be actions which prevent the element from matching
         * @return
         */
        public boolean isRegexComplete();

}
