package de.mbutscher.wikiandpad.parsing;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Regex extends Token implements NecessaryRegexProvider {

    protected String pattern;
    protected int flags;
    protected int flagsMask;
    private Pattern re;
    
    public Regex(String pattern, int flags, int flagsMask) {
        this.pattern = pattern;
        this.flags = flags;
        if (flagsMask == -1)
            this.flagsMask = flags;
        else
            this.flagsMask = flagsMask;
        
        re = java.util.regex.Pattern.compile(pattern, flags);
    }

    public Regex(String pattern, int flags) {
        this(pattern, flags, -1);
    }
    
    public Pattern getRegex() {
        return re;
    }

    public int getRegexFlagsMask() {
        return flagsMask;
    }

    public boolean isRegexComplete() {
        // TODO Auto-generated method stub
        return true;
    }

    @Override
    protected ParsingResult parseImpl(String instring, int loc,
            ParsingState state, boolean doActions) throws ParseException {
        
        /*if ( 1 > (instring.length() - loc)) {
            Log.d("WikidPad", "test");
        }*/
        
        Matcher mt = re.matcher(instring.substring(loc));
        if (!mt.lookingAt()) {
            throw new ParseException(instring, loc,
                    String.format("Pattern '%s' doesn't match",
                    re.pattern()), this);
        }
        
        int end = mt.end() + loc;
        
        ParsingResult parsingResult = new ParsingResult(end,
                SyntaxNode.buildSyntaxNode(mt.group()));
        
        return parsingResult;
    }

    @Override
    public String toString() {
        return String.format("Re:(%s)", pattern);
    }
}
