package de.mbutscher.wikiandpad.parsing;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.mbutscher.wikiandpad.utils.StringOps;

public abstract class SyntaxNode {

    protected int pos;
    protected int strLength=-1;
    protected String name;
    protected HashMap<String, Object> dict;
    

    public SyntaxNode(int pos, String name) {
        this.dict = new HashMap<String, Object>();        
        this.name = name;
        this.pos = pos;
    }


    public HashMap<String, Object> getDict() {
        return dict;
    }

    public boolean dContainsKey(String key) {
        return dict.containsKey(key);
    }

    public Object dGet(String key) {
        return dict.get(key);
    }

    /*
    @SuppressWarnings("unchecked")
    public <T> T dGet(String key, T defValue) {
        return (T) dict.get(key);
    }
    */

    public SyntaxNode dPut(String key, Object value) {
        dict.put(key, value);
        return this;
    }


    public int getPos() {
        return pos;
    }

    public void setPos(int pos) {
        this.pos = pos;
    }


    public String getName() {
        return name;
    }
    
    public void setName(String name) {
        this.name = name;
    }

    public int getStrLength() {
        if (strLength == -1)        
            recalcStrLength();
        
        return strLength;
    }
    
    /**
     * @return true iff this is a TerminalNode
     */
    public abstract boolean isTerminal();

    /**
     * @return contents of node as list of SyntaxNode s
     */
    public abstract List<SyntaxNode> asList();

    /**
     * @return NonTerminalNode for this node (or itself if it is already one)
     */
    public abstract NonTerminalNode asNonTerminal();

    public abstract List<String> asStringList(String sep);

    public List<String> asStringList() {
        return asStringList("");
    }

    public abstract String getString();

    public abstract void recalcStrLength();

    public abstract List<SyntaxNode> findNodesForCharPos(int charPos);

    /**
     * For TerminalNodes returns itself, for nonterminal nodes returns
     * the first contained {@link SyntaxNode} or null if no contained
     * nodes available
     * @return
     */
    public abstract SyntaxNode getFirstToken();
    
    public NonTerminalNode getFirstTokenNt() {
        return (NonTerminalNode) getFirstToken();
    }

    public TerminalNode getFirstTokenT() {
        return (TerminalNode) getFirstToken();
    }

    protected abstract void pprintRecurs(int ind, int inc, StringBuilder result);

    public String pprint(int ind, int inc) {
        StringBuilder result = new StringBuilder();
        pprintRecurs(ind, inc, result);
        return result.toString();
    }
   
    public String pprint() {
        return pprint(0, 2);
    }
    
    @Override
    public String toString() {
        return pprint();
    }
   
    public static String pprintList(List<SyntaxNode> lst) {
        StringBuilder result = new StringBuilder();
        result.append("[\n");
        for(SyntaxNode item: lst) {
            item.pprintRecurs(0, 2, result);
            result.append("\n");
        }

        result.append("]");
       
        return result.toString();
    }

    public String pprintDict() {
        StringBuilder result = new StringBuilder();
        result.append("{");
        for(Map.Entry<String, Object> entry: dict.entrySet()) {
            result.append(String.format("'%s': %s,", entry.getKey(),
                    entry.getValue()));
        }
        result.append("}");
        
        return result.toString();
    }

    public static TerminalNode buildSyntaxNode(String sub, int pos,
            String name) {
        return new TerminalNode(sub, pos, name);
    }

    public static SyntaxNode buildSyntaxNode(SyntaxNode sub, int pos,
            String name) {
        if (pos != -1)
            sub.setPos(pos);
        if (StringOps.notEmpty(name))
            sub.setName(name);
        
        return sub;
    }

    public static NonTerminalNode buildSyntaxNode(List<SyntaxNode> sub, int pos,
            String name) {
        return new NonTerminalNode(sub, pos, name);
    }

    
    public static TerminalNode buildSyntaxNode(String sub, int pos) {
        return buildSyntaxNode(sub, pos, null);
    }

    public static TerminalNode buildSyntaxNode(String sub) {
        return buildSyntaxNode(sub, -1, null);
    }

    public static SyntaxNode buildSyntaxNode(SyntaxNode sub, int pos) {
        return buildSyntaxNode(sub, pos, null);
    }

    public static SyntaxNode buildSyntaxNode(SyntaxNode sub) {
        return sub;
    }

    public static NonTerminalNode buildSyntaxNode(List<SyntaxNode> sub, int pos) {
        return buildSyntaxNode(sub, pos, null);
    }

    public static NonTerminalNode buildSyntaxNode(List<SyntaxNode> sub) {
        return buildSyntaxNode(sub, -1, null);
    }

}
