package de.mbutscher.wikiandpad.utils;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Stack;

public class CompatibleFilenameIterator implements Iterator<String> {
    
    protected String baseName;
    protected String suffix;
    protected boolean asciiOnly;
    protected int maxLength;
    protected int randomLength;

    protected int step = 0;
    protected Stack<String> baseQuoted;
    protected int overallLength;
    protected String beforeRandom;

    
    public static final String ESCAPING_CHARACTERS = "%@~";

    public static final String FORBIDDEN_CHARACTERS =
            ":/\\*?\"'<>|;![]" + ESCAPING_CHARACTERS;
    public static final HashSet<Character> FORBIDDEN_CHARACTERS_SET =
            StringOps.stringToSet(FORBIDDEN_CHARACTERS);

    public static final String FORBIDDEN_START = FORBIDDEN_CHARACTERS + ".$ -";
    public static final HashSet<Character> FORBIDDEN_START_SET =
            StringOps.stringToSet(FORBIDDEN_START);

    
    public boolean hasNext() {
        return true;
    }

    public void remove() {
        throw new UnsupportedOperationException();
    }

   
    public CompatibleFilenameIterator(String baseName, String suffix,
            boolean asciiOnly, int maxLength, int randomLength) {

        this.baseName = baseName;
        this.suffix = suffix;
        this.asciiOnly = asciiOnly;
        this.maxLength = maxLength;
        this.randomLength = randomLength;
        
        baseQuoted = new Stack<String>();
    
        if (baseName.length() > 0) {
            char c = baseName.charAt(0);
    
            if (c < 32 || FORBIDDEN_START_SET.contains(c) ||
                    (asciiOnly && c > 127))
                baseQuoted.push(StringOps.strQuoteChar(c));
            else
                baseQuoted.push(Character.toString(c));
            
            for(int i = 1; i < baseName.length(); ++i) {
                c = baseName.charAt(i);
    
                if (c < 32 || FORBIDDEN_CHARACTERS_SET.contains(c) ||
                        (asciiOnly && c > 127))
                    baseQuoted.push(StringOps.strQuoteChar(c));
                else
                    baseQuoted.push(Character.toString(c));
            }
        }
        
        overallLength = 0;
        for(String s: baseQuoted) {
            overallLength += s.length();
        }
        
        overallLength += suffix.length();
        
        while (overallLength > maxLength) {
            overallLength -= baseQuoted.pop().length();
        }
    }
    
    public CompatibleFilenameIterator(String baseName, String suffix,
            boolean asciiOnly) {
        this(baseName, suffix, asciiOnly, 120, 10);
    }
    
    protected String step0() {
        if (baseName.length() > 0) {
            step = 1;
            // First try, no random part
            return StringOps.strJoin(baseQuoted) + suffix;
        }
        else {
            step = 1;
            return step1();
        }
    }
    
    protected String step1() {
        // Add random part to length
        overallLength += 1 + randomLength;
        
        // Shorten baseQuoted again
        while (overallLength > maxLength)
            overallLength -= baseQuoted.pop().length();
        
        beforeRandom = StringOps.strJoin(baseQuoted) + "~";
        
        // Now we try MD5-Hash. This is one last try to create a filename which
        // is non-ambigously connected to the baseName
        
        String hashStr  = StringOps.getMd5B36ByString(baseName, randomLength);
        
        step = 2;
        return beforeRandom + hashStr + suffix;
    }
    
    protected String stepMore() {
        // Now build infinite random names
        return beforeRandom + StringOps.createRandomString(randomLength) +
                suffix;
    }

    public String next() {
        switch (step) {
        case 0: return step0();
        case 1: return step1();
        default: return stepMore();
        }
    }
    
    

}

/*
   




*/
