package de.mbutscher.wikiandpad.utils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class FileUtils {

    public static class RelativeFilePathAndContained {
        public String filePath;
        public boolean contained;

        public RelativeFilePathAndContained(String filePath, boolean contained) {
            this.filePath = filePath;
            this.contained = contained;
        }
    }
    
    /**
     *     Cut an absolute path into all of its pieces, through
    all path components up to the name of the file (if any).
    Returns an array of the elements, last element may be
    empty string.
     * @param path
     * @return
     */
    public static List<String> splitpath(String path) {
        path = (new File(path)).getAbsolutePath();
        
        if (path.length() == 0 || path.charAt(0) != '/')
            return Arrays.asList(new String[0]);
        
        return Arrays.asList(path.substring(1).split("/"));
    }


    /**
     * Returns a relative (if possible) path to address the file
    toFilePath if you are in directory location as first tuple item.


    Function returns None as first tuple item if an absolute path is needed!
    
    Tests if toFilePath is a file or dir contained in location and returns
        truth value in second tuple item

    Both parameters should be normalized with os.path.abspath
     * @param relativeTo Directory where you are
     * @param fullPath absolute path to file you want to reach
     * @return
     */
    public static RelativeFilePathAndContained 
            getRelativeFilePathAndTestContained(String relativeTo,
            String fullPath) {
        
        if (relativeTo == null || "".equals(relativeTo))
            return new RelativeFilePathAndContained(null, false);
        
        ArrayList<String> locParts = new ArrayList<String>(
                splitpath(relativeTo));
        
        if ("".equals(locParts.get(locParts.size() - 1)))
            locParts.remove(locParts.size() - 1);
        
        int locLen = locParts.size();
        ArrayList<String> fileParts = new ArrayList<String>(
                splitpath(fullPath));
        
        for(int i = 0; i < locLen; ++i) {
            if (fileParts.size() == 0)
                break; // TODO: Error ???
            if(!locParts.get(0).equals(fileParts.get(0)))
                break;

            locParts.remove(0);
            fileParts.remove(0);
        }
        
        ArrayList<String> result = new ArrayList<String>();
        
        /*
        
        if (locParts.size() == locLen) {
            // Nothing matches at all, absolute path needed
            return new RelativeFilePathAndContained(null, false);
        }
        
        */
        
        boolean isContained = fileParts.size() > 0;
        if (locParts.size() > 0) {
            // go back some steps
            for(int i = 0; i < locParts.size(); ++i)
                result.add("..");
            isContained = false;
        }
        
        result.addAll(fileParts);
        
        if (result.size() == 0)
            return new RelativeFilePathAndContained("", false);
        else
            return new RelativeFilePathAndContained(StringOps.strJoin(
                    File.separator, result), isContained);
    }
    
    public static String join(String... parts) {
        if (parts.length == 0)
            return "";
        if (parts.length == 1)
            return parts[0];
        
        // Find an absolute part and delete relative parts before it
        
        int lastAbs;
        for(lastAbs = parts.length - 1; lastAbs >= 0; --lastAbs) {
            if (parts[lastAbs].startsWith(File.separator))
                break;
        }
        
        if (lastAbs == parts.length - 1)
            return parts[lastAbs];
        
        boolean resultAbs;
        if (lastAbs == -1) {
            resultAbs = false;
            lastAbs = 0;
        }
        else {
            resultAbs = true;
            parts[lastAbs] = parts[lastAbs].substring(1);
        }

        ArrayList<String> result = new ArrayList<String>(Arrays.asList(
                parts[lastAbs].split(File.separator)));

        for(int i = lastAbs + 1; i < parts.length; ++i) {
            ArrayList<String> other = new ArrayList<String>(Arrays.asList(
                    parts[i].split(File.separator)));
            
            while (other.size() > 0 && "..".equals(other.get(0))) {
                if (result.size() > 0) {
                    result.remove(result.size() - 1);
                    other.remove(0);
                }
                else
                    return null;
            }
            
            result.addAll(other);
        }
        
        if (resultAbs)
            return File.separator + StringOps.strJoin(File.separator, result);
        else
            return StringOps.strJoin(File.separator, result);
    }
    
    
    /**
     * Returns path relative to relativeTo. If relativeTo is "" or None
    the path is absolute.
     * @param relativeTo
     * @param fullPath
     * @return
     */
    public static String getRelativeFilePathIfPossible(String relativeTo,
            String fullPath) {

        if (relativeTo == null || "".equals(relativeTo))
            return fullPath;
        
        String relPath = getRelativeFilePathAndTestContained(relativeTo,
                fullPath).filePath;
        
        if (relPath == null)
            return fullPath;
        
        return relPath;
    }

    
    /**
     * Specialized function. Creates a file in directory path, fills it with 
    content (byte string), closes it and returns its full path.
     * @param content
     * @param suffix
     * @param path
     * @param relativeTo path relative to which the path should be or None
        for absolute path
     * @param textMode Convert lineEndings?
     * @return
     * @throws IOException 
     */
    public static String createTempFile(byte[] content, String suffix,
            String path, String relativeTo, boolean textMode)
            throws IOException {
        /* if (path == null)
            path = getDefaultTempFilePath() */
        
        File tempFile;
        
        if (path != null)       
            tempFile = File.createTempFile("wkap", suffix, new File(path));
        else
            tempFile = File.createTempFile("wkap", suffix);
        
        FileOutputStream fos = new FileOutputStream(tempFile);
        try {
            fos.write(content);
        }
        catch (IOException e) {
            // try to remove created temp file
            tempFile.delete();
            throw e;
        }
        fos.close();

        return getRelativeFilePathIfPossible(relativeTo,
                tempFile.getAbsolutePath());
    }


    public static String createTempFile(String content, String suffix,
            String path, String relativeTo, boolean textMode)
            throws IOException {
        
        byte[] byteContent;
        
        if (content.length() == 0 || content.charAt(0) != '\ufeff')
            byteContent = ("\ufeff" + content).getBytes("UTF-8");
        else
            byteContent = content.getBytes("UTF-8");
        
        return createTempFile(byteContent, suffix, path, relativeTo, textMode);
    }
    
    
    
}



/*



def createTempFile(content, suffix, path=None, relativeTo=None, textMode=False):
    if path is None:
        path = getDefaultTempFilePath()
    
    fd, fullPath = tempfile.mkstemp(suffix=pathEnc(suffix), dir=pathEnc(path),
            text=textMode)
    try:
        try:
            if isinstance(content, unicode):
                # assert textMode
                content = content.encode("utf-8")
                os.write(fd, BOM_UTF8)
                os.write(fd, content)
            elif isinstance(content, str):
                os.write(fd, content)
            else:    # content is a sequence
                try:
                    iCont = iter(content)
        
                    firstContent = iCont.next()
                    
                    unic = False
                    if isinstance(firstContent, unicode):
                        firstContent = firstContent.encode("utf-8")
                        os.write(fd, BOM_UTF8)
                        unic = True
    
                    assert isinstance(firstContent, str)
                    os.write(fd, firstContent)
    
                    while True:
                        content = iCont.next()
    
                        if unic:
                            assert isinstance(content, unicode)
                            content = content.encode("utf-8")
    
                        assert isinstance(content, str)
                        os.write(fd, content)
                except StopIteration:
                    pass
        finally:
            os.close(fd)
    except Exception, e:
        traceback.print_exc()
        # Something went wrong -> try to remove temporary file
        try:
            os.unlink(fullPath)
        except:
            traceback.print_exc()
        
        raise e


    return getRelativePath(relativeTo, fullPath)



*/