package de.mbutscher.wikiandpad.utils;

public class ThreadHolder extends BasicThreadStop {

    protected Thread thread;
    
    public synchronized Thread getThread() {
        return thread;
    }

    public synchronized void setThread(Thread thread) {
        this.thread = thread;
    }
    
    //protected setAsThreadStopForThisThread()
    
    /**
     * Creates a new thread for a Runnable and holds this thread. When thread
     * starts it automatically sets {@link ThreadStop#currentThreadStop} to
     * the started thread.
     * @param r Runnable to run
     * @param daemon Should execute as daemon?
     */
    public synchronized void prepare(final Runnable rb, boolean daemon) {
        Runnable actualRunnable = new Runnable() {
            public void run() {
                synchronized (ThreadHolder.this) {
                    if (!ThreadHolder.this.isValidThread())
                        return;
                    ThreadStop.currentThreadStop.set(ThreadHolder.this);
                }
                rb.run();
            }
        };

        Thread t = new Thread(actualRunnable);

        t.setDaemon(daemon);
        t.setPriority(Thread.NORM_PRIORITY - 1);
        setThread(t);
    }
    
    /**
     * Same as prepare(r, false)
     * @see #prepare(Runnable, boolean)
     */
    public void prepare(Runnable r) {
        prepare(r, false);
    }

    /**
     * Calls {@link Thread#start()} on the hold thread.
     */
    public synchronized void start() {
        thread.start();
    }
    
    /**
     * Executes a Runnable in a new thread and holds this thread 
     * @param r Runnable to run
     * @param daemon Should execute as daemon?
     */
    public synchronized void execute(Runnable r, boolean daemon) {
        prepare(r, daemon);
        start();
    }
    
    /**
     * Same as execute(r, false)
     * @see #execute(Runnable, boolean)
     */
    public void execute(Runnable r) {
        execute(r, false);
    }
    
    @Override
    public boolean isValidThread(Thread thread) {
        return this.thread == thread;
    }
}
