package de.mbutscher.wikiandpad.utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

/**
 * General utilities
 * @author Michael Butscher
 *
 */
public class Utilities {
    public static <T> IterableIterator<T> wrapIterable(final Iterator<T> iterator) {
        return new IterableIterator<T>() {
            public Iterator<T> iterator() {
                return iterator;
            }

            public boolean hasNext() {
                return iterator.hasNext();
            }

            public T next() {
                return iterator.next();
            }

            public void remove() {
                iterator.remove();
            }
        };
    }

    public static Object[] extractColumn(Object[][] tab, int column) {
        Object[] result = new Object[tab.length];
        for(int i = 0; i < tab.length; ++i)
            result[i] = tab[i][column];
        
        return result;
    }
    
    public static List<Object> extractColumn(List<Object[]> tab, int column) {
        Object[] result = new Object[tab.size()];
        for(int i = 0; i < result.length; ++i)
            result[i] = tab.get(i)[column];
        
        return Arrays.asList(result);
    }

    
    @SuppressWarnings("unchecked")
    public static <T> List<T> castList(List<?> list) {
        List<T> result = new ArrayList<T>(list.size());
        
        for(int i = 0; i < list.size(); ++i)
            result.add((T)list.get(i));

        return result;
    }

    @SuppressWarnings("unchecked")
    public static <T> List<T> castList(Object[] array) {
        List<T> result = new ArrayList<T>(array.length);
        
        for(int i = 0; i < array.length; ++i)
            result.add((T)array[i]);

        return result;
    }
    
    public static <T> List<T> multiple(T item, int times) {
        List<T> result = new ArrayList<T>(times);
        for(int i = 0; i < times; ++i) {
            result.add(item);
        }
        
        return result;
    }
    


    public static Character[] castArrayNonPrimitive(char[] list) {
        Character[] result = new Character[list.length];
        
        for(int i = 0; i < list.length; ++i)
            result[i] = list[i];

        return result;
    }
    
    public static <T> HashSet<T> buildHashSet(T... items) {
        return new HashSet<T>(Arrays.asList(items));
    }

    
    public static boolean isCurrentValidThread() {
        ThreadStop current = ThreadStop.currentThreadStop.get();
        if (current == null)
            return true;
        
        return current.isValidThread();
    }

    public static void testCurrentValidThread()
            throws NotCurrentThreadException {
        ThreadStop current = ThreadStop.currentThreadStop.get();
        if (current == null)
            return;
        
        current.testValidThread();
    }
    
    public <T> T nullToDef(T val, T def) {
        return val != null ? val : def; 
    }
        


}
