package org.folder;

import java.io.File;
import java.io.FileFilter;
import java.text.Collator;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;

import de.mbutscher.wikiandpad.WikiAndPadApplication;
import de.mbutscher.wikiandpad.alphabeta.R;
import de.mbutscher.wikiandpad.utils.MiscEvent;
import de.mbutscher.wikiandpad.utils.MiscEventSource;
import de.mbutscher.wikiandpad.utils.ui.ArrayListAdapter;

import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.graphics.Canvas;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.LayerDrawable;
import android.os.Environment;
import android.os.SystemClock;
import android.util.StateSet;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.view.animation.AlphaAnimation;
import android.view.animation.Animation;
import android.view.animation.LayoutAnimationController;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TabHost;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;

public class FolderPicker extends Dialog implements
        OnItemClickListener, OnClickListener, MiscEventSource {

    private ListView mFoldersListView;
    protected ListView lruFilesListView;
    private TextView mCurrentFolder;
    private Folder mPath;
    private Folder mFilePath;
    private File mRootSDCard;
    private FolderAdapter mFolderAdapter;
    protected RecentFileListAdapter lruFilesAdapter;
    protected TabHost tabHost;
    
    private OnClickListener mListener;
    private boolean mAcceptFiles;
    private View mOkButton;
    protected boolean pressedOk=false;
    
    protected final int ID_CLEAR_LRU_FILES = 1;

    public FolderPicker(Context context, OnClickListener listener, int themeId) {
        this(context, listener, themeId, false, null);
    }

    public FolderPicker(Context context, OnClickListener listener, int themeId,
            boolean acceptFiles, List<String> recentFilePaths) {
        super(context, themeId);
        mListener = listener;

        setContentView(R.layout.folders);
        
        tabHost = (TabHost) findViewById(android.R.id.tabhost);
        tabHost.setup();
        
        tabHost.addTab(tabHost.newTabSpec("browse").setContent(R.id.tab_browse)
                .setIndicator("Browse"));
        
        tabHost.addTab(tabHost.newTabSpec("lru").setContent(R.id.tab_lru)
                .setIndicator("Recent"));

        mOkButton = findViewById(R.id.ok_btn);

        mAcceptFiles = acceptFiles;
        if (acceptFiles) {
            setTitle(R.string.pick_file);
            mOkButton.setVisibility(View.GONE);
        }
        else {
            setTitle(R.string.pick_folder);
            mOkButton.setOnClickListener(this);
        }
        
        SharedPreferences pref = WikiAndPadApplication.getAppPrefs();
        String confRoot = pref.getString("filePicker_rootDirectory", "");
        
        if ("".equals(confRoot))
            mRootSDCard = Environment.getExternalStorageDirectory();
        else {
            mRootSDCard = new File(confRoot);
            if (!mRootSDCard.isDirectory())
                mRootSDCard = Environment.getExternalStorageDirectory();
        }

        findViewById(R.id.cancel_btn).setOnClickListener(this);
        mCurrentFolder = (TextView) findViewById(R.id.current_folder);
        mCurrentFolder.setSelected(true);
        mFoldersListView = (ListView) findViewById(R.id.folders);
        mFoldersListView.setOnItemClickListener(this);

        Animation animation = new AlphaAnimation(0, 1);
        animation.setDuration(250);
        LayoutAnimationController controller = new LayoutAnimationController(
                animation);
        mFoldersListView.setLayoutAnimation(controller);
        
        mFolderAdapter = new FolderAdapter();
        mFoldersListView.setAdapter(mFolderAdapter);

        lruFilesAdapter = new RecentFileListAdapter(recentFilePaths);
        lruFilesListView = (ListView) findViewById(R.id.lruFiles);
        lruFilesListView.setAdapter(lruFilesAdapter);
        
        lruFilesListView.setOnItemClickListener(this);

        
        mPath = new Folder(mRootSDCard.getAbsolutePath());
        updateAdapter();
        
    }

    public String getPath() {
        if (!mAcceptFiles) {
            return mPath.getAbsolutePath();
        } else
        if (mFilePath != null) {
            return mFilePath.getAbsolutePath();
        } else {
            return null;
        }
    }
    
    public boolean isPressedOk() {
        return pressedOk;
    }

    public void onClick(View v) {
        if (v == mOkButton) {
            pressedOk = true;
            if (mListener != null) {
                mListener.onClick(this, DialogInterface.BUTTON_POSITIVE);
            }
        }
        else
            pressedOk = false;

        dismiss();
    }
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        menu.add(Menu.NONE, ID_CLEAR_LRU_FILES, Menu.NONE, "Clear Recent");
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        return tabHost.getCurrentTabTag() == "lru";
        // return super.onPrepareOptionsMenu(menu);
    }

    @Override
    public boolean onMenuItemSelected(int featureId, MenuItem item) {
        switch (item.getItemId()) {
        case ID_CLEAR_LRU_FILES:
            WikiAndPadApplication.cloneMainEvent(this)
                    .put("request clear lruWikiPaths").send();

            lruFilesAdapter.clear();
            return true;
        }
        
        return super.onMenuItemSelected(featureId, item);
    }


    public static Comparator<File> FILE_COMPARATOR = new Comparator<File>() {

        protected final Collator COLLATOR = Collator.getInstance();
        
        public int compare(File object1, File object2) {
            return COLLATOR.compare(object1.toString(), object2.toString());
        }
    };

    private void updateAdapter() {
        mCurrentFolder.setText(mPath.getAbsolutePath());
        mFolderAdapter.clear();
        if (!mPath.equals(mRootSDCard)) {
            mFolderAdapter.add(new Folder(mPath, true));
        }
        File[] dirs = mPath.listFiles(mDirFilter);
        // File[] dirs = mPath.listFiles();
        Arrays.sort(dirs, FILE_COMPARATOR);
        for (int i = 0; i < dirs.length; i++) {
            mFolderAdapter.add(new Folder(dirs[i]));
        }
        if (mAcceptFiles) {
            File[] files = mPath.listFiles(mFileFilter);
            Arrays.sort(files, FILE_COMPARATOR);
            for (int i = 0; i < files.length; i++) {
                mFolderAdapter.add(new Folder(files[i]));
            }
        }
        mFolderAdapter.notifyDataSetChanged();
        mFoldersListView.setSelection(0);
        mFoldersListView.startLayoutAnimation();
    }

    public void onItemClick(AdapterView<?> parent, View view, int position,
            long id) {

        if (parent == mFoldersListView) {
            if (mAcceptFiles) {
                Folder item = (Folder) mFolderAdapter.getItem(position);
                if (item.isDirectory()) {
                    mPath = item;
                    updateAdapter();
                    mFilePath = null;
                } else {
                    mCurrentFolder.setText(item.getAbsolutePath());
                    mFilePath = item;
                    pressedOk = true;
                    if (mListener != null) {
                        mListener.onClick(this, DialogInterface.BUTTON_POSITIVE);
                    }
                    dismiss();
                }
            } else {
                mPath = (Folder) mFolderAdapter.getItem(position);
                updateAdapter();
            }
        }
        else if (parent == lruFilesListView) {
            Folder item = lruFilesAdapter.getItem(position);

            if (item.isDirectory()) {
                mPath = item;
                mFilePath = null;
            }
            else {
                mFilePath = item;
            }
            pressedOk = true;
            if (mListener != null) {
                mListener.onClick(this, DialogInterface.BUTTON_POSITIVE);
            }
            dismiss();
        }
        else {
            return;
        }
    }

    private FileFilter mDirFilter = new FileFilter() {
        public boolean accept(File file) {
            return file.isDirectory();
        }
    };

    private FileFilter mFileFilter = new FileFilter() {
        public boolean accept(File file) {
            return file.isFile();
        }
    };
    
    class FolderAdapter extends BaseAdapter {
        ArrayList<Folder> mFolders = new ArrayList<Folder>();
        LayoutInflater mInflater = LayoutInflater.from(getContext());
        private Drawable[] mFolderUpLayers;
        private Drawable[] mFolderLayers;
        private Drawable mFileDrawable;
        
        public FolderAdapter() {
            Resources res = getContext().getResources();
            mFolderUpLayers = new Drawable[] {
                    res.getDrawable(R.drawable.ic_launcher_folder_up_closed),
                    res.getDrawable(R.drawable.ic_launcher_folder_up_open),
            };
            mFolderLayers = new Drawable[] {
                    res.getDrawable(R.drawable.ic_launcher_folder_closed),
                    res.getDrawable(R.drawable.ic_launcher_folder_open),
            };
            mFileDrawable = res.getDrawable(R.drawable.file);
        }
        
        public int getCount() {
            return mFolders.size();
        }

        public void add(Folder folder) {
            mFolders.add(folder);
        }

        public void clear() {
            mFolders.clear();
        }

        public Object getItem(int position) {
            return mFolders.get(position);
        }

        public long getItemId(int position) {
            return position;
        }

        public View getView(int position, View convertView, ViewGroup parent) {
            View v = mInflater.inflate(R.layout.folder, parent, false);
            Folder folder = mFolders.get(position);
            TextView name = (TextView) v.findViewById(R.id.folder_name);

            Drawable drawable = null;
            if (folder.isParent) {
                name.setText("[..]");
                drawable = new FolderTransitionDrawable(mFolderUpLayers);
            } else {
                name.setText(folder.getName());
                if (folder.isDirectory()) {
                    drawable = new FolderTransitionDrawable(mFolderLayers);
                } else {
                    drawable = mFileDrawable;
                }
            }
            v.findViewById(R.id.folder_icon).setBackgroundDrawable(drawable);
            return v;
        }
    }
    
    
    static class FolderTransitionDrawable extends LayerDrawable implements Runnable {
        private static final int TRANSITION_DURATION = 10;
        private static final int[] STATE_SELECTED = {android.R.attr.state_selected};
        private static final int[] STATE_PRESSED = {android.R.attr.state_pressed};
        
        private boolean mActive;
        private int mAlpha;
        private int mFrom;
        private int mTo;
        private long mStartTime;
        private long mEndTime;

        public FolderTransitionDrawable(Drawable[] layers) {
            super(layers);
            mAlpha = 255;
        }
        
        
        @Override
        public boolean isStateful() {
            return true;
        }
        
        @Override
        protected boolean onStateChange(int[] state) {
            boolean active = StateSet.stateSetMatches(STATE_SELECTED, state) | StateSet.stateSetMatches(STATE_PRESSED, state);
            if (active != mActive) {
                mActive = active;
//                Log.d("FolderTransitionDrawable", "onStateChange " + StateSet.dump(state) + " " + active);
                if (!active) {
                    unscheduleSelf(this);
                    if (mAlpha != 255) {
                        startTransition(false);
                    }
                } else {
                    scheduleSelf(this, SystemClock.uptimeMillis() + 500);
                }
                return true;
            }
            return false;
        }

        public void run() {
            startTransition(true);
        }

        private void startTransition(boolean showLayer1) {
            mStartTime = SystemClock.uptimeMillis();
            mFrom = mAlpha;
            mEndTime = mStartTime;
            if (showLayer1) {
                mTo = 0;
                mEndTime += mAlpha * TRANSITION_DURATION / 255;
            } else {
                mTo = 255;
                mEndTime += (255 - mAlpha) * TRANSITION_DURATION / 255;
            }
            invalidateSelf();
        }

        @Override
        public void draw(Canvas canvas) {
            boolean done = true;
            
            if (mStartTime != 0) {
                long time = SystemClock.uptimeMillis();
                done = time > mEndTime;
                if (done) {
                    mStartTime = 0;
                    mAlpha = mTo;
                } else {
                    float normalized = (time - mStartTime) / (float) (mEndTime - mStartTime);
                    mAlpha = (int) (mFrom  + (mTo - mFrom) * normalized);
                }
            }
            
            Drawable d = getDrawable(0);
            d.setAlpha(mAlpha);
            d.draw(canvas);
            
            d = getDrawable(1);
            d.setAlpha(255 - mAlpha);
            d.draw(canvas);
            
            if (!done) {
                invalidateSelf();
//                Log.d("TAG", "draw invalidate");
            }
        }
    }
    
    @SuppressWarnings("serial")
    class Folder extends File {
        protected boolean isParent;

        public Folder(File file) {
            super(file.getAbsolutePath());
        }

        public Folder(File file, boolean unused) {
            super(file.getParent());
            isParent = true;
        }
        
        public Folder(String path) {
            super(path);
        }
    }
    
    protected class RecentFileListAdapter extends ArrayListAdapter<Folder> {
        public RecentFileListAdapter(List<String> recentFilePaths) {
            super(FolderPicker.this.getContext(), R.layout.big_little_list_item);
            
            if (recentFilePaths != null) {
                for (String path: recentFilePaths) {
                    add(new Folder(path));
                }
                notifyDataSetChanged();
            }
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            // TODO: Reuse existing view

            View result = convertView;
            if (result == null) {
                result = View.inflate(getContext(),
                        R.layout.big_little_list_item, null);
            }
                
            File entry = getItem(position);
            
            TextView tv = (TextView) result.findViewById(R.id.text1);
            tv.setText(entry.getName());
            
            tv = (TextView) result.findViewById(R.id.text2);
            tv.setText(entry.getParent());

            return result;
        }
    }
    
    protected MiscEvent miscEvent = new MiscEvent(this);
    
    public MiscEvent getMiscEvent() {
        return miscEvent;
    }

}
